using System;
using System.Text;

namespace RepPP.Toolkit {
	/// <summary>
	/// TypedInstanceGenForVB:  Typed instance generator for visual basic.
	/// </summary>
	public class TypedInstanceGenForVB : TypedInstanceGenBase {
#if VC71
            private const string            m_strRem = "'";
#else
            private const string            m_strRem = "'''";
#endif

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public TypedInstanceGenForVB() : base(new Web.WebServicesGenForVB()) {
		}

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedGroupInstance:      Generate typed instance for a group
        /// </summary>
        /// <param name="cgrp">                 Card group</param>
        /// <param name="grp">                  Group</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strNameSpace">         Namespace</param>
        /// <param name="strBaseClassName">     Name of the base class</param>
        /// <param name="iLevel">               Right pad level</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************        
		private string IntGenerateTypedGroupInstance(RepPP.CardGroup   cgrp,
		                                             RepPP.Group       grp,
                                                     string            strCrdClassPrefix,
                                                     string            strGrpClassPrefix,
                                                     string            strGrpPropertyPrefix,
                                                     string            strFldPropertyPrefix,
                                                     string            strNameSpace,
                                                     string            strBaseClassName,
		                                             int               iLevel,
		                                             bool              bGenWS) {
            StringBuilder           strBuilder;
            Web.WebServicesGenBase  wsGen;
            string                  strGrpName;
            string                  strGrpPropName;
            string                  strClassName;
            string                  strFldPropName;
            string                  strChildClassName;
            string                  strContainerClassName;
            string                  strGroupBaseClassName;
            string                  strIndent;
            string                  strFldType;
            int                     iCount;
            int                     iIndex;
            string                  strUserClassName;
            
            wsGen                 = WSGenerator;
            strIndent             = "".PadRight(iLevel * 4, ' ');
            strBuilder            = new StringBuilder(null, 4096);        
            iCount                = grp.Fields.Count;
            strClassName          = DecorateGroupClassName(TYPED_GROUP_INST_PREFIX + strGrpClassPrefix, ((cgrp != null) ? GetCardGroupName(cgrp) : GetGroupName(grp)));
            strGroupBaseClassName = (strBaseClassName != String.Empty) ? strBaseClassName : "RepPP.TypedGroupInstanceBase";
            strUserClassName      = DecorateGroupClassName(strGrpClassPrefix, (cgrp != null) ? GetCardGroupName(cgrp) : GetGroupName(grp));
            strBuilder.Append(strIndent + m_strRem + " <summary>\r\n");
            strBuilder.Append(strIndent + m_strRem + " Typed Instance of group " + grp.Name + "\r\n");
            strBuilder.Append(strIndent + m_strRem + " </summary>\r\n");
            if (cgrp == null) {
                strBuilder.Append(strIndent + "<RepPP.TypedInstanceClass(\"" + grp.Name + "\", 100)> _\r\n");
            }
            strBuilder.Append(strIndent + "Public Class " + strClassName + "\r\n");
            strBuilder.Append(strIndent + "    Inherits " + strGroupBaseClassName + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " <summary>" + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " Class constructor.\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " <param name=\"pobj\">    Reference to the internal object</param>\r\n");
            strBuilder.Append(strIndent + "    Public Sub New(pobj As Int32)\r\n");
            if (cgrp == null) {
                strBuilder.Append(strIndent + "        MyBase.New(pobj, _\r\n");
                strBuilder.Append(strIndent + "               New string() { _\r\n");
                iIndex = 0;
                foreach (RepPP.Field fld in grp.Fields) {
                    iIndex = iIndex + 1;
                    strBuilder.Append(strIndent + "                                                \"" + fld.Name + "\"" +
                                        (iIndex != iCount ? ", _" : " _") + "\r\n");
                }
                strBuilder.Append(strIndent + "                                               })\r\n");
            } else {
                strBuilder.Append(strIndent + "        MyBase.New(pobj)\r\n");
            }
            strBuilder.Append(strIndent + "    End Sub\r\n");
            if (cgrp == null) {
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " Create:               Create a new instance.\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <param name=\"app\">    Rep++ application object</param>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <returns>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " New instance\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " </returns>\r\n");
                strBuilder.Append(strIndent + "    Public Shared Function Create(app As RepPP.Application) As " + strUserClassName + "\r\n");
                strBuilder.Append(strIndent + "        Return CType(app.Groups(\"" + grp.Name + "\").GroupInstances.Create(RepPP.GroupType.sdHorizontal), " + strUserClassName + ")\r\n");
                strBuilder.Append(strIndent + "    End Function\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " Create:               Create a new instance of the specified type.\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <param name=\"app\">    Rep++ application object</param>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <param name=\"type\">   Type of the wrapper class to create</param>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " <returns>\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " New instance\r\n");
                strBuilder.Append(strIndent + "    " + m_strRem + " </returns>\r\n");
                strBuilder.Append(strIndent + "    Protected Shared Function Create(ByVal app As RepPP.Application, type As Type) As " + strUserClassName + "\r\n");
                strBuilder.Append(strIndent + "        Dim giRetVal        As RepPP.GroupInstance\r\n");
                strBuilder.Append(strIndent + "        Dim grp             As RepPP.Group\r\n");
                strBuilder.Append(strIndent + "        Dim typeOld         As Type\r\n");
                strBuilder.Append(strIndent + "        Dim typedInstFact   As RepPP.TypedInstanceFactory\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append(strIndent + "        grp             = app.Groups(\"" + grp.Name + "\")\r\n");
                strBuilder.Append(strIndent + "        typedInstFact   = CType(app.GetObjFactory(GetType(RepPP.GroupInstance)), RepPP.TypedInstanceFactory)\r\n");
                strBuilder.Append(strIndent + "        typeOld         = typedInstFact.GetTypeAssociateWithGroup(grp)\r\n");
                strBuilder.Append(strIndent + "        typedInstFact.AddGroup(grp, type)\r\n");
                strBuilder.Append(strIndent + "        Try\r\n");
                strBuilder.Append(strIndent + "            giRetVal = grp.GroupInstances.Create(RepPP.GroupType.sdHorizontal)\r\n");
                strBuilder.Append(strIndent + "        Finally\r\n");
                strBuilder.Append(strIndent + "            typedInstFact.AddGroup(grp, typeOld)\r\n");
                strBuilder.Append(strIndent + "        End Try\r\n");
                strBuilder.Append(strIndent + "        Return CType(giRetVal, " + strUserClassName + ")\r\n");
                strBuilder.Append(strIndent + "    End Function\r\n");
                foreach (RepPP.Field fld in grp.Fields) {
                    strFldPropName  = DecorateFieldPropertyName(strFldPropertyPrefix, GetFieldName(fld));
                    strFldType      = fld.GetType().Name.Replace("Field", "FieldInstance");
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " " + fld.Prompt + "\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
                    strBuilder.Append(strIndent + "    Public ReadOnly Property " + strFldPropName + " As RepPP." + strFldType + "\r\n");
                    strBuilder.Append(strIndent + "        Get\r\n");
                    strBuilder.Append(strIndent + "            Return(CType(FieldInstances(\"" + fld.Name + "\"), RepPP." + strFldType + "))\r\n");
                    strBuilder.Append(strIndent + "        End Get\r\n");
                    strBuilder.Append(strIndent + "    End Property\r\n");
                }
                if (bGenWS) {
                    strContainerClassName = (cgrp == null) ? strClassName : DecorateCardClassName(strCrdClassPrefix, GetCardName(cgrp.Card));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateGroupSerializationFunctions(strContainerClassName, cgrp, grp, iLevel * 4 + 4));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateGroupClass(grp, iLevel * 4 + 4, "", strGrpClassPrefix));
                }
            } else {
                foreach (RepPP.CardGroup cgrpChild in cgrp.ChildCardGroups) {
                    strGrpName          = cgrpChild.Group.Name;
                    strChildClassName   = DecorateGroupClassName(strGrpClassPrefix, GetCardGroupName(cgrpChild));
                    strGrpPropName      = DecorateGroupPropertyName(strGrpPropertyPrefix, GetCardGroupName(cgrpChild));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " <summary>\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " Gets the wrapper for the instance of group " + strGrpName + "\r\n");
                    strBuilder.Append(strIndent + "    " + m_strRem + " </summary>" + "\r\n");
                    strBuilder.Append(strIndent + "    Public ReadOnly Property " + strGrpPropName + " As " + strChildClassName + "\r\n");
                    strBuilder.Append(strIndent + "        Get \r\n");
                    strBuilder.Append(strIndent + "            Dim ginstChild As RepPP.GroupInstance\r\n");
                    strBuilder.Append("\r\n");
                    strBuilder.Append(strIndent + "            ginstChild = CType(IIf(CurrentLine <> -1, ChildGroupInstances(\"" + strGrpName + "\"), Nothing), RepPP.GroupInstance)\r\n");
                    strBuilder.Append(strIndent + "            Return(CType(ginstChild, " + strChildClassName + "))\r\n");
                    strBuilder.Append(strIndent + "        End Get\r\n");
                    strBuilder.Append(strIndent + "    End Property\r\n");
                }
                foreach (RepPP.CardGroup cgrpChild in cgrp.ChildCardGroups) {
		            strBuilder.Append(IntGenerateTypedGroupInstance(cgrpChild,
		                                                            cgrpChild.Group,
                                                                    strCrdClassPrefix,
                                                                    strGrpClassPrefix,
                                                                    strGrpPropertyPrefix,
                                                                    strFldPropertyPrefix,
                                                                    strNameSpace,
                                                                    strNameSpace + "." + DecorateGroupClassName(strGrpClassPrefix, GetGroupName(cgrpChild.Group)),
		                                                            iLevel + 1,
		                                                            bGenWS));
                }
            }
            strBuilder.Append(strIndent + "End Class ' class " + strClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateGroupInstanceProperty:      Generate the group instances properties
        /// </summary>
        /// <param name="cgrp">                 Card group</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <returns>
        /// The properties code
        /// </returns>
        //
        //******************************************************************
        private string GenerateGroupInstanceProperty(RepPP.CardGroup    cgrp,
                                                     string             strGrpClassPrefix,
                                                     string             strGrpPropertyPrefix) {
            StringBuilder   strBuilder;
            string          strGroupClassName;
            string          strGrpName;
            string          strGrpPropName;

            strBuilder          = new StringBuilder(null, 4096);            
            strGrpName          = cgrp.Group.Name;
            strGrpPropName      = DecorateGroupPropertyName(strGrpPropertyPrefix, GetCardGroupName(cgrp));
            strGroupClassName   = DecorateGroupClassName(strGrpClassPrefix, GetGroupName(cgrp.Group));
            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + " Gets the wrapper for the instance of group " + strGrpName + "\r\n");
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        Public ReadOnly Property " + strGrpPropName + " As " + strGroupClassName + "\r\n");
            strBuilder.Append("            Get\r\n");
            if (cgrp.Parent == null) {
                strBuilder.Append("                Return(CType(RootGroupInstance, " + strGroupClassName + "))\r\n");
            } else {
                strBuilder.Append("                Return(CType(GetCurrentGroupInstance(Card.FindCardGroup(\"" + strGrpName + "\")), " + strGroupClassName + "))\r\n");
            }
            strBuilder.Append("            End Get\r\n");
            strBuilder.Append("        End Property\r\n");
            foreach (RepPP.CardGroup cgrpChild in cgrp.ChildCardGroups) {
                strBuilder.Append(GenerateGroupInstanceProperty(cgrpChild, strGrpClassPrefix, strGrpPropertyPrefix));
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// IntGenerateTypedCardInstance:       Generate typed instance for a card
        /// </summary>
        /// <param name="crd">                  Card</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strCrdBaseClassName">  Name of the base class for the card</param>
        /// <param name="strGrpBaseClassName">  Name of the base class for the groups</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		private string IntGenerateTypedCardInstance(RepPP.Card  crd, 
                                                    string      strCrdClassPrefix,
                                                    string      strGrpClassPrefix,
                                                    string      strGrpPropertyPrefix,
                                                    string      strFldPropertyPrefix,
                                                    string      strCrdBaseClassName,
                                                    string      strGrpBaseClassName,
                                                    bool        bGenWS) {
            StringBuilder           strBuilder;
            Web.WebServicesGenBase  wsGen;
            RepPP.CardGroup         cgrpRoot;
            string                  strCardClassName;
            string                  strBaseClassName;
            string                  strUserCardClassName;
            
            wsGen                = WSGenerator;
            strBuilder           = new StringBuilder(null, 4096);         
            strCardClassName     = DecorateCardClassName(TYPED_CARD_INST_PREFIX + strCrdClassPrefix, GetCardName(crd));
            strBaseClassName     = (strCrdBaseClassName != String.Empty) ? strCrdBaseClassName : "RepPP.TypedCardInstanceBase";
            strUserCardClassName = DecorateCardClassName(strCrdClassPrefix, GetCardName(crd));
            strBuilder.Append("    " + m_strRem + " <summary>\r\n");
            strBuilder.Append("    " + m_strRem + " Typed Instance of card " + crd.Name + "\r\n");
            strBuilder.Append("    " + m_strRem + " </summary>\r\n");
            strBuilder.Append("    <RepPP.TypedInstanceClass(\"" + crd.Name + "\", 100)> _\r\n");
            strBuilder.Append("    Public Class " + strCardClassName + "\r\n");
            strBuilder.Append("        Inherits " + strBaseClassName + "\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + " Class constructor.\r\n");
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        " + m_strRem + " <param name=\"pobj\">    Reference to the internal object</param>\r\n");
            strBuilder.Append("        Public Sub New(pobj As Int32)\r\n");
            strBuilder.Append("            MyBase.New(pobj)\r\n");
            strBuilder.Append("        End Sub\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + " Create:               Create a new instance.\r\n");
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        " + m_strRem + " <param name=\"app\">    Rep++ application object</param>\r\n");
            strBuilder.Append("        " + m_strRem + " <returns>\r\n");
            strBuilder.Append("        " + m_strRem + " New instance\r\n");
            strBuilder.Append("        " + m_strRem + " </returns>\r\n");
            strBuilder.Append("        Public Shared Function Create(app As RepPP.Application) As " + strUserCardClassName + "\r\n");
            strBuilder.Append("            Return CType(app.Cards(\"" + crd.Name + "\").CardInstances.Add(), " + strUserCardClassName + ")\r\n");
            strBuilder.Append("        End Function\r\n");

            strBuilder.Append("\r\n");
            strBuilder.Append("        " + m_strRem + " <summary>\r\n");
            strBuilder.Append("        " + m_strRem + " Create:                     Create a new instance of the specified type.\r\n");
            strBuilder.Append("        " + m_strRem + " </summary>\r\n");
            strBuilder.Append("        " + m_strRem + " <param name=\"app\">          Rep++ application object</param>\r\n");
            strBuilder.Append("        " + m_strRem + " <param name=\"typeWrapper\">  Type of the wrapper class to create</param>\r\n");
            strBuilder.Append("        " + m_strRem + " <returns>\r\n");
            strBuilder.Append("        " + m_strRem + " New instance\r\n");
            strBuilder.Append("        " + m_strRem + " </returns>\r\n");
            strBuilder.Append("        Public Shared Function Create(app As RepPP.Application, typeWrapper As System.Type) As " + strUserCardClassName + "\r\n");
            strBuilder.Append("            Dim ciRetVal      As RepPP.CardInstance\r\n");
            strBuilder.Append("            Dim crd           As RepPP.Card\r\n");
            strBuilder.Append("            Dim typeOld       As System.Type\r\n");
            strBuilder.Append("            Dim typedInstFact As RepPP.TypedInstanceFactory\r\n");
            strBuilder.Append("\r\n");
            strBuilder.Append("            crd             = app.Cards(\"" + crd.Name + "\")\r\n");
            strBuilder.Append("            typedInstFact   = CType(app.GetObjFactory(GetType(RepPP.CardInstance)), RepPP.TypedInstanceFactory)\r\n");
            strBuilder.Append("            typeOld         = typedInstFact.GetTypeAssociateWithCard(crd)\r\n");
            strBuilder.Append("            typedInstFact.AddCard(crd, typeWrapper)\r\n");
            strBuilder.Append("            Try \r\n");
            strBuilder.Append("                ciRetVal = crd.CardInstances.Add()\r\n");
            strBuilder.Append("            Finally\r\n");
            strBuilder.Append("                typedInstFact.AddCard(crd, typeOld)\r\n");
            strBuilder.Append("            End Try\r\n");
            strBuilder.Append("            Return CType(ciRetVal, " + strUserCardClassName + ")\r\n");
            strBuilder.Append("        End Function\r\n");
            cgrpRoot = crd.ChildCardGroups[0];
            if (cgrpRoot != null) {
                strBuilder.Append(GenerateGroupInstanceProperty(cgrpRoot, strGrpClassPrefix, strGrpPropertyPrefix));
                if (bGenWS) {
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateCardSerializationFunctions(strCardClassName, crd, 8));
                    strBuilder.Append("\r\n");
                    strBuilder.Append(wsGen.GenerateCardClass(crd, 8, "", "", strGrpClassPrefix));
                }
            }
            strBuilder.Append("    End Class ' class " + strCardClassName + "\r\n");
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedGroupInstance:     Generate user typed instance for a group
        /// </summary>
        /// <param name="grp">                  Group</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedGroupInstance(RepPP.Group   grp, 
                                                              string        strGrpClassPrefix,
                                                              string        strNameSpace,
                                                              string        strRootNameSpace) {
            StringBuilder   strBuilder;
            bool            bGenNamesapce;
            string          strClassName;
            string          strBaseClassName;
            string          strIndent = "    ";

            bGenNamesapce = (strNameSpace != String.Empty);
            strBuilder    = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateSignature("'"));
            strBuilder.Append("Imports System\r\n");
            strBuilder.Append("\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("Namespace " + strNameSpace + "\r\n");
            }
            strClassName     = DecorateGroupClassName(strGrpClassPrefix, GetGroupName(grp));
            strBaseClassName = DecorateGroupClassName(TYPED_GROUP_INST_PREFIX + strGrpClassPrefix, GetGroupName(grp));
            strBuilder.Append(strIndent + m_strRem + " <summary>\r\n");
            strBuilder.Append(strIndent + m_strRem + " Typed Instance of group " + grp.Name + "\r\n");
            strBuilder.Append(strIndent + m_strRem + " </summary>\r\n");
            strBuilder.Append(strIndent + "<RepPP.TypedInstanceClass(\"" + grp.Name + "\", 200)> _\r\n");
            strBuilder.Append(strIndent + "Public Class " + strClassName + "\r\n");
            strBuilder.Append(strIndent + "    Inherits " + strBaseClassName + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " <summary>" + "\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " Class constructor.\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " </summary>\r\n");
            strBuilder.Append(strIndent + "    " + m_strRem + " <param name=\"pobj\">    Reference to the internal object</param>\r\n");
            strBuilder.Append(strIndent + "    Public Sub New(pobj As Int32)\r\n");
            strBuilder.Append(strIndent + "        MyBase.New(pobj)\r\n");
            strBuilder.Append(strIndent + "    End Sub\r\n");
            strBuilder.Append(strIndent + "End Class ' class " + strClassName + "\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("End Namespace ' namespace " + strNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedGroupInstance:         Generate typed instance for a group
        /// </summary>
        /// <param name="grp">                  Group</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <param name="strBaseClassName">     Name of the base class</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// The generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedGroupInstance(RepPP.Group   grp, 
                                                          string        strCrdClassPrefix,
                                                          string        strGrpClassPrefix,
                                                          string        strGrpPropertyPrefix,
                                                          string        strFldPropertyPrefix,
                                                          string        strNameSpace,
                                                          string        strRootNameSpace,
                                                          string        strBaseClassName,
                                                          bool          bGenWS) {
            StringBuilder   strBuilder;
            bool            bGenNamesapce;

            bGenNamesapce = (strNameSpace != String.Empty);
            strBuilder    = new StringBuilder(null, 4096);            
            strBuilder.Append(GenerateSignature("'"));
            strBuilder.Append("Imports System\r\n");
            strBuilder.Append("\r\n");
            if (bGenNamesapce) {
                strBuilder.Append("Namespace " + strNameSpace + "\r\n");
            }
            strBuilder.Append(IntGenerateTypedGroupInstance(null,
                                                            grp,
                                                            strCrdClassPrefix,
                                                            strGrpClassPrefix,
                                                            strGrpPropertyPrefix,
                                                            strFldPropertyPrefix,
                                                            strNameSpace,
                                                            strBaseClassName,
                                                            1, 
                                                            bGenWS));
            if (bGenNamesapce) {
                strBuilder.Append("End Namespace ' namespace " + strNameSpace + "\r\n");
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateUserTypedCardInstance:     Generate user typed instance for a card
        /// </summary>
        /// <param name="crd">                  Card</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated card class name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateUserTypedCardInstance(RepPP.Card     crd, 
                                                             string         strCrdClassPrefix,
                                                             string         strNameSpace,
                                                             string         strRootNameSpace) {
            StringBuilder   strBuilder;
            RepPP.CardGroup cgrpRoot;
            bool            bGenNamesapce;
            string          strCardClassName;
            string          strBaseClassName;

            bGenNamesapce = (strNameSpace != String.Empty);
            strBuilder    = new StringBuilder(null, 4096);            
            if (crd.ChildCardGroups.Count != 0) {
                cgrpRoot = crd.ChildCardGroups[0];
                strBuilder.Append(GenerateSignature("'"));
                strBuilder.Append("Imports System" + "\r\n");
                strBuilder.Append("\r\n");
                if (bGenNamesapce) {
                    strBuilder.Append("Namespace " + strNameSpace + "\r\n");
                }
                strCardClassName  = DecorateCardClassName(strCrdClassPrefix, GetCardName(crd));
                strBaseClassName  = DecorateCardClassName(TYPED_CARD_INST_PREFIX + strCrdClassPrefix, GetCardName(crd));
                strBuilder.Append("    " + m_strRem + " <summary>\r\n");
                strBuilder.Append("    " + m_strRem + " Typed Instance of card " + crd.Name + "\r\n");
                strBuilder.Append("    " + m_strRem + " </summary>\r\n");
                strBuilder.Append("    <RepPP.TypedInstanceClass(\"" + crd.Name + "\", 200)> _\r\n");
                strBuilder.Append("    Public Class " + strCardClassName + "\r\n");
                strBuilder.Append("        Inherits " + strBaseClassName + "\r\n");
                strBuilder.Append("\r\n");
                strBuilder.Append("        " + m_strRem + " <summary>\r\n");
                strBuilder.Append("        " + m_strRem + " Class constructor.\r\n");
                strBuilder.Append("        " + m_strRem + " </summary>\r\n");
                strBuilder.Append("        " + m_strRem + " <param name=\"pobj\">    Reference to the internal object</param>\r\n");
                strBuilder.Append("        Public Sub New(pobj As Int32)\r\n");
                strBuilder.Append("            MyBase.New(pobj)\r\n");
                strBuilder.Append("        End Sub\r\n");
                strBuilder.Append("    End Class ' class " + strCardClassName + "\r\n");
                if (bGenNamesapce) {
                    strBuilder.Append("End Namespace ' namespace " + strNameSpace + "\r\n");
                }
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerateTypedCardInstance:          Generate typed instance for a card
        /// </summary>
        /// <param name="crd">                  Card</param>
        /// <param name="strCrdClassPrefix">    Prefix for generated card class name</param>
        /// <param name="strGrpClassPrefix">    Prefix for generated group class name</param>
        /// <param name="strGrpPropertyPrefix"> Prefix for generated group property name</param>
        /// <param name="strFldPropertyPrefix"> Prefix for generated field property name</param>
        /// <param name="strNameSpace">         The namespace</param>
        /// <param name="strRootNameSpace">     The root namespace of the project</param>
        /// <param name="strCrdBaseClassName">  Name of the base class for the card instance</param>
        /// <param name="strGrpBaseClassName">  Name of the base class for the group instance</param>
        /// <param name="bGenWS">               true to generate Web Services</param>
        /// <returns>
        /// Generated class
        /// </returns>
        //
        //******************************************************************
		public override string GenerateTypedCardInstance(RepPP.Card     crd, 
                                                         string         strCrdClassPrefix,
                                                         string         strGrpClassPrefix,
                                                         string         strGrpPropertyPrefix,
                                                         string         strFldPropertyPrefix,
                                                         string         strNameSpace,
                                                         string         strRootNameSpace,
                                                         string         strCrdBaseClassName,
                                                         string         strGrpBaseClassName,
                                                         bool           bGenWS) {
            StringBuilder   strBuilder;
            RepPP.CardGroup cgrpRoot;
            bool            bGenNamesapce;

            bGenNamesapce = (strNameSpace != String.Empty);
            strBuilder    = new StringBuilder(null, 4096);            
            if (crd.ChildCardGroups.Count != 0) {
                cgrpRoot = crd.ChildCardGroups[0];
                strBuilder.Append(GenerateSignature("'"));
                strBuilder.Append("Imports System" + "\r\n");
                strBuilder.Append("\r\n");
                if (bGenNamesapce) {
                    strBuilder.Append("Namespace " + strNameSpace + "\r\n");
                }
                strBuilder.Append(IntGenerateTypedCardInstance(crd, 
                                                               strCrdClassPrefix,
                                                               strGrpClassPrefix,
                                                               strGrpPropertyPrefix,
                                                               strFldPropertyPrefix,
                                                               strCrdBaseClassName,
                                                               strGrpBaseClassName,
                                                               bGenWS));
                if (bGenNamesapce) {
                    strBuilder.Append("End Namespace ' namespace " + strNameSpace + "\r\n");
                }
            }
            return(strBuilder.ToString());
        }

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all cards and groups which need to be regenerated
		/// </summary>
		/// <param name="app">                  Rep++ application object</param>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		public override void RegenerateTypedInstance(RepPP.Application app, ILog log, IProjectFolder projFolder, bool bFullSync) {
		    RegenerateTypedInstance(app, log, projFolder, ".vb", bFullSync);
		}

        //******************************************************************
        //
		/// <summary>
		/// RegenerateTypedInstance:            Regenerates all cards and groups which need to be regenerated
		/// </summary>
        /// <param name="log">                  Log object for name clash. Null for no name clash check</param>
		/// <param name="projFolder">           Interface with the project folder</param>
		/// <param name="bFullSync">            true to fully synchronize type. false to only sync the existing one</param>
        //
        //******************************************************************
		public override void RegenerateTypedInstance(ILog log, IProjectFolder projFolder, bool bFullSync) {
		    RegenerateTypedInstance(null, log, projFolder, ".vb", bFullSync);
		}
	} // class TypedInstanceGenForVB
} // namespace
