using System;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using System.ComponentModel;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Implement a DropDownList control which is attached to the Rep++ field meta-data
    /// </summary>
    //
    //********************************************************************************
	[ToolboxData("<{0}:FldDropDownList runat=server></{0}:FldDropDownList>")]
	[DefaultProperty("SelectedIndex")]
	public class FldDropDownList : System.Web.UI.WebControls.DropDownList, IFldEditableControl {
	    /// <summary>Field validator attached to this control</summary>
	    protected FldValidator          m_fldValidator;
	    /// <summary>Field control utility class</summary>
	    protected FldControlUtil        m_fldControlUtil;

        //********************************************************************************
        //
		/// <summary>
		/// ctor:               Class constructor
		/// </summary>
		//
        //********************************************************************************
		public FldDropDownList() {
		    m_fldControlUtil    = new FldControlUtil(this, true);
		    ValidatorPosition   = FldValidator.AutoPos.After;
		}

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_fldValidator != null) {
                m_fldValidator.Dispose();
            }
            if (m_fldControlUtil != null) {
                m_fldControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// Refresh:        Refresh the list of items
		/// </summary>
		//
        //********************************************************************************
        public void Refresh() {
            ListItemCollection items;

            items = base.Items;
            if (items != null) {
                items.Clear();
                FillItemList(items);
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            if (base.Items != null) {
                base.Items.Clear();
            }
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            if (base.Items != null) {
                base.Items.Clear();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the DropDownList
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]
        [Editor(typeof(FldControlUtil.FldControlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the DropDownList")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Field:              Field attached to the DropDownList
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		    set {
		        m_fldControlUtil.Field = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// ID:         ID of the control
        /// </summary>
        //
        //********************************************************************************
        public override string ID {
            get {
                return base.ID;
            }
            set {
                base.ID = value;
                if (m_fldValidator != null) {
                    m_fldValidator.ControlToValidate = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ValidatorPosition:      Position of the automatic validator
        /// </summary>
        //
        //********************************************************************************
        [Category("Layout")]
        [Bindable(true)]
        [DefaultValue(FldValidator.AutoPos.After)]
        [Description("Position of the automatic validator")]
        public virtual FldValidator.AutoPos ValidatorPosition {
            get {
                return((m_fldValidator != null) ? m_fldValidator.AutoPosition : FldValidator.AutoPos.None);
            }
            set {
                if (value == FldValidator.AutoPos.None) {
                    if (m_fldValidator != null) {
                        m_fldValidator.Dispose();
                        m_fldValidator = null;
                    }
                } else {
                    if (m_fldValidator == null) {
                        m_fldValidator                   = new FldValidator(this);
                        m_fldValidator.ControlToValidate = ID;
                    }
                    m_fldValidator.AutoPosition = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Render:                 Render the control
        /// </summary>
        /// <param name="writer">   HTML writer</param>
        //
        //********************************************************************************
        protected override void Render(HtmlTextWriter writer) {
            int iDummy;

            if (Visible) {
                // Force the reset of the selected index
                iDummy = SelectedIndex;
                base.Render(writer);
                if (m_fldValidator != null) {
                    if (Field != null) {
                        m_fldValidator.InitFromFather(this);
                        m_fldValidator.RenderControl(writer);
                    } else {
                        m_fldValidator.Dispose();
                        m_fldValidator = null;
                    }
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataMember:         DataMember... We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataMember {
            get {
                return(base.DataMember);
            }
            set {
                base.DataMember = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataSource:         Datasource... We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override object DataSource {
            get {
                return(base.DataSource);
            }
            set {
                base.DataSource = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataTextField:      We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataTextField {
            get {
                return(base.DataTextField);
            }
            set {
                base.DataTextField = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// DataValueField:     We must not support it....
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string DataValueField {
            get {
                return(base.DataValueField);
            }
            set {
                base.DataValueField = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FillItemList:           Fill the item list with the choice list content
        /// </summary>
        /// <param name="items">    List of items</param>
        //
        //********************************************************************************
        private void FillItemList(ListItemCollection items) {
            Field       fld;
            int         iIndex;
            ChoiceList  choiceList;
            string      strValue = String.Empty;
            string      strCode  = String.Empty;
            string      strDesc  = String.Empty;
            bool        bShowDesc;
			ListItem	listItem;

            fld = m_fldControlUtil.Field;
            if (fld != null && base.Items.Count == 0) {
                choiceList  = fld.ChoiceList;
                bShowDesc   = ShowDescription;
                iIndex      = 0;
                do {
                    iIndex = choiceList.ExtractChoice(iIndex, ref strValue, ref strCode, ref strDesc);
                    if (iIndex != -1) {
					    listItem       = new ListItem();
                        if (strValue.Equals(String.Empty)) {
    					    listItem.Value = strCode;
                        } else {
	    				    listItem.Value = strValue;
                        }
                        if (bShowDesc && strDesc != String.Empty) {
                            listItem.Text = strDesc;
                        } else {
                            listItem.Text = strCode;
                        }
                        items.Add(listItem);
                    }
                } while (iIndex != -1);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnSelectedIndexChanged:     Called when the selected index changes
        /// </summary>
        /// <param name="e">            Event parameter</param>
        //
        //********************************************************************************
        protected override void OnSelectedIndexChanged(EventArgs e) {
            if (Visible) {
                this.SelectedIndex = base.SelectedIndex;
                base.OnSelectedIndexChanged(e);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// OnDataBinding:      Bind the value to the field
        /// </summary>
        /// <param name="e">    Event parameter</param>
        //
        //********************************************************************************
        protected override void OnDataBinding(EventArgs e) {
            int     iDummy;
            
            base.OnDataBinding (e);
            // Force the reset of the selected index
            iDummy = SelectedIndex;
        }

        //********************************************************************************
        //
        /// <summary>
        /// Items:                  List of items in the list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override ListItemCollection Items {
            get {
                ListItemCollection  itemsRetVal;
                
                itemsRetVal = base.Items;
                if (itemsRetVal != null) {
                    FillItemList(itemsRetVal);
                }
                return(itemsRetVal);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// SelectedIndex
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        [Description("Currently selected item")]
        public override int SelectedIndex {
            get {
                int         iRetVal;
                Field       fld;
                string      strValue;
                
                fld = m_fldControlUtil.Field;
                if (fld == null) {
                    iRetVal = base.SelectedIndex;
                } else {
                    if (AttachedLine > -1) {
                        strValue = fld.Group.ActiveGroupInstance.GetFieldValue(fld, AttachedLine, false);
                    } else {
                        strValue = fld.Value;
                    }
                    iRetVal = fld.ChoiceList.FindValue(strValue);
                    if (iRetVal != base.SelectedIndex) {
                        base.SelectedIndex = iRetVal;
                    }
                }
                return(iRetVal);
            }
            set {
                Field       fld;
                string      strValue = String.Empty;
                
                fld = m_fldControlUtil.Field;
                if (fld != null && fld.ChoiceList != null) {
                    if (value >= 0 && value < fld.ChoiceList.Count) {
                        strValue = Items[value].Value;
                    }
                    if (AttachedLine > -1) {
                        fld.Group.ActiveGroupInstance.SetFieldValue(fld, 
                                                                    AttachedLine,
                                                                    false,
                                                                    RepPP.SetFieldOpt.sdSetNormal,
                                                                    strValue);
                    } else {
                        fld.Value = strValue;
                    }
                }
                if (base.SelectedIndex != value) {
                    base.SelectedIndex = value;
                }
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ShowDescription:    Determine if descriptions or codes are shown in the list
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(true)]
        [Bindable(false)]
        [Description("Determine if descriptions or codes are shown in the list")]
        public virtual bool ShowDescription {
            get {
                bool    bRetVal;
                object  obj;
                
                obj = ViewState["ShowDescription"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = true;
                }
                return(bRetVal);
            }
            set {
                ViewState["ShowDescription"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Enabled:       Determines if the control is enabled
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Enabled {
            get {
                bool    bRetVal;
                Field   fld;
                object  obj;
                
                obj = ViewState["Enabled"];
                if (obj != null) {
                    bRetVal = (bool)obj;
                } else {
                    bRetVal = true;
                }
                if (bRetVal) {
                    fld = m_fldControlUtil.Field;
                    bRetVal = (fld == null) ? true : fld.IsSelectable;
                }
                return(bRetVal);
            }
            set {
                ViewState["Enabled"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Visible:        Determines if the control is visible
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override bool Visible {
            get {
                bool     bRetVal;
                Field    fld;
                
                fld = m_fldControlUtil.Field;
                bRetVal = (fld == null || m_fldControlUtil.IsDesignMode) ? true : fld.IsVisible;
                return(bRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AttachedLine:             Attached group instance line
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        [DefaultValue(-1)]
        [Description("Attached group instance line index")]
		public virtual int AttachedLine {
            get {
                int    iRetVal;
                object  obj;
                
                obj = ViewState["AttachedLine"];
                if (obj != null) {
                    iRetVal = (int)obj;
                } else {
                    iRetVal = -1;
                }
                return(iRetVal);
            }
            set {
                ViewState["AttachedLine"] = value;
            }
		}
    } // Class
} // Namespace
