//*********************************************************
//
// FldListBox:		    Overload the microsoft .net Listbox to implement 
//                      rep++ Listbox field control
//
// Author:		        Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.ComponentModel.Design;
using RepPP.Toolkit;
using RepPP;

namespace RepPP.Toolkit.Window {
	/// <summary>
	/// FldListBox: Create a Listbox around a rep++ field
	/// </summary>
	public class FldListBox : System.Windows.Forms.ListBox, FldControlUtil.IFldControl {
        /// <summary>Field control utility class</summary>
        protected WndFldControlUtil                 m_fldControlUtil;
        /// <summary>True if filling the control</summary>
        private bool                                m_bFilling;
		/// <summary> Required designer variable</summary>
		private System.ComponentModel.Container     components = null;
        //*********************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //*********************************************************
		public FldListBox() {
			InitializeComponent();
            m_bFilling               = false;
            m_fldControlUtil         = new WndFldControlUtil(this, false);
		}

        //*********************************************************	    
        //
        /// <summary>
        /// Dispose:                    Dispose the editor
        /// </summary>
        /// <param name="bDisposing">   true if dispose, false if GC </param>
        //
        //*********************************************************	    
        protected override void Dispose(bool bDisposing) {
            RepPP.Application   app;

			if (bDisposing) {
                if(components != null) {
                    components.Dispose();
                }
                if (m_fldControlUtil != null) {
                    app = Application;
                    if (app != null) {
                        app.ProgInit   -= new RepPP.ProgInitEventHandler(this.OnProgInit);
                        app.ProgEnd    -= new RepPP.ProgEndEventHandler(this.OnProgEnd);
                        app.FldChange  -= new RepPP.FldChangeEventHandler(this.OnFldChange);
                    }
                    m_fldControlUtil.Dispose();
                    m_fldControlUtil = null;
                }
			}
			base.Dispose(bDisposing);
		}

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
            Items.Clear();
            BuildListItems();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
            Items.Clear();
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_fldControlUtil != null) {
                m_fldControlUtil.TriggerComponentChange();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FieldFullName:              Full name of the field attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [Editor(typeof(FldControlUtil.FldControlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the field attached to the label")]
		public FldFullName FieldFullName {
		    get {
		        return(m_fldControlUtil.FieldFullName);
		    }
		    set {
		        m_fldControlUtil.FieldFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public RepPP.Application Application {
		    get {
		        return(m_fldControlUtil.Application);
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// FieldName:      Get and set the field name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string FieldName {
            get {
                return(FieldFullName.FieldName);
            }
            set {
                FieldFullName.FieldName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GroupName:      Get and set the group name
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
        public string GroupName {
            get {
                return(FieldFullName.GroupName);
            }
            set {
                FieldFullName.GroupName = value;
                Invalidate();
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// Field:              Get the attached field
        /// </summary>
        //
        //*********************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Field Field {
		    get {
		        return(m_fldControlUtil.Field);
		    }
		    set {
		        m_fldControlUtil.Field = value;
		    }
		}

        //*********************************************************
        //
        /// <summary>
        /// InitializeCtlv:     Initialize the control
        /// </summary>
        //
        //*********************************************************
        private void InitializeCtlv() {
            string          strValue;
            string          strOneValue;
            int             iIndex;
            RepPP.Field     fld;

            fld  = m_fldControlUtil.Field;
            if (fld != null) {
                this.Enabled   = fld.IsSelectable;
                this.Visible   = fld.IsVisible;
                if (fld.Type == RepPP.FieldType.sdFieldMultiAlphaChoice ||
                    fld.Type == RepPP.FieldType.sdFieldMultiAlphaChoice) {
                    this.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
                }
                BuildListItems();
                if (fld.IsChoiceAvailable) {
                    m_bFilling = true;
                    if (fld.Type == RepPP.FieldType.sdFieldMultiAlphaChoice ||
                        fld.Type == RepPP.FieldType.sdFieldMultiAlphaChoice) {
                        strValue = fld.Value;
                        iIndex   = strValue.IndexOf(",", 0, strValue.Length);
                        while(iIndex != -1) {
                            strOneValue = strValue.Substring(0, iIndex); 
                            strValue    = strValue.Substring(iIndex + 1, strValue.Length - iIndex - 1);
                            this.SelectedIndex = fld.ChoiceList.FindValue(strOneValue);
                            iIndex = strValue.IndexOf(",", 0, strValue.Length);
                        }
                        this.SelectedIndex = fld.ChoiceList.FindValue(strValue);
                    } else {
                        this.SelectedIndex = fld.ChoiceList.FindValue(fld.Value);
                    }
                    m_bFilling = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// GetMaxCodeWidth:        Get the max width of the list codes
        ///                         measured using the current font
        /// </summary>
        /// <param name="lst">      Choice list</param>
        /// <returns>
        /// Max code width
        /// </returns>
        //
        //*********************************************************
        protected virtual int GetMaxCodeWidth(ChoiceList lst) {
            int         iRetVal  = 0;
            int         iIndex   = 0;
            string      strCode  = null;
            string      strValue = null;
            string      strDesc  = null;
            int         iCodeWidth;
            Graphics    gr;

            gr      = CreateGraphics();
            do {
                iIndex = lst.ExtractChoice(iIndex, ref strValue, ref strCode, ref strDesc);
                if (iIndex != -1) {
                    iCodeWidth = (int)gr.MeasureString(strCode, this.Font).Width;
                    if (iCodeWidth > iRetVal) {
                        iRetVal = iCodeWidth;
                    }
                }
            } while (iIndex != -1);
            return(iRetVal);
        }

        //*********************************************************
        //
        /// <summary>
        /// BuildListItemsv:        Build the combobox list items
        /// </summary>
        //
        //*********************************************************
        protected virtual void BuildListItems() {
            int         iIndex;
            int         iMaxCodeWidth;
            int         iSpaceWidth;
            int         iCodeWidth;
            string      strCode  = null;
            string      strDesc  = null;
            string      strValue = null;
            string      strChoiceAns;
            Field       fld;
            Graphics    gr;
            ChoiceList  lst;
            int         iCount;

            Items.Clear();
            fld = m_fldControlUtil.Field;
            if (fld != null && fld.IsChoiceAvailable == true) {
                lst             = fld.ChoiceList;
                iCount          = lst.Count;
                gr              = CreateGraphics();
                iSpaceWidth     = (int)gr.MeasureString(" ", Font).Width;
                iMaxCodeWidth   = GetMaxCodeWidth(lst) + iSpaceWidth * 4;
                iIndex          = 0;
                do {
                    iIndex = lst.ExtractChoice(iIndex, ref strValue, ref strCode, ref strDesc);
                    if (iIndex != -1) {
                        if (strDesc != String.Empty) {
                            iCodeWidth   = (int)gr.MeasureString(strCode, Font).Width; 
                            strChoiceAns = strCode.PadRight(strCode.Length + (iMaxCodeWidth - iCodeWidth)/iSpaceWidth);
                            strChoiceAns = strChoiceAns + strDesc;
                        } else {
                            strChoiceAns = strCode;
                        }
                        this.Items.Add(strChoiceAns);
                    }
                } while (iIndex != -1);
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// SetFieldValuev:     Set the field value
        /// </summary>
        //
        //*********************************************************
        private void SetFieldValuev() {
            string      strCode;
            string      strValue = "";
            RepPP.Field fld;
            
            fld = m_fldControlUtil.Field;
            if (fld != null && fld.IsChoiceAvailable) {
                if (fld.Type == RepPP.FieldType.sdFieldMultiAlphaChoice ||
                    fld.Type == RepPP.FieldType.sdFieldMultiAlphaChoice) {
                    for(int iIndex = 0; iIndex < this.SelectedIndices.Count; iIndex ++) {
                        strCode = this.Items[iIndex].ToString();
                        if (strCode.Length > fld.MaxSize) {
                            strCode = strCode.Substring(0, fld.MaxSize);
                        }
                        if (strValue == "") {
                            strValue = fld.ChoiceList.ConvertCodeToValue(strCode);
                        } else {
                            strValue = strValue + "," + fld.ChoiceList.ConvertCodeToValue(strCode);
                        }
                    }
                } else {            
                    strCode = this.Text;
                    if (strCode.Length > fld.MaxSize) {
                        strCode = strCode.Substring(0, fld.MaxSize);
                    }
                    strValue = fld.ChoiceList.ConvertCodeToValue(strCode);
                }
                if (fld.Value != strValue) {
                    m_bFilling  = true;
                    fld.Value   = strValue;
                    m_bFilling  = false;
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgInit:           Handle the application initialized event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgInit(object sender, RepPP.ApplicationEventArgs e) {
            this.Items.Clear();
            InitializeCtlv();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnProgEnd:            Handle the application end event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnProgEnd(object sender, RepPP.ApplicationEventArgs e) {
            Field = null;
            this.Items.Clear();
        }

        //*********************************************************
        //
        /// <summary>
        /// OnFldChange:          Handle the field change event
        /// </summary>
        /// <param name="sender"> Sender object   </param>
        /// <param name="e">      Event arguments </param>
        //
        //*********************************************************
        private void OnFldChange(object sender, RepPP.FieldGenEventArgs e) {
            Field   fld;

            fld = (m_fldControlUtil != null) ? m_fldControlUtil.Field : null;
            if (fld != null && fld == e.Field && !m_bFilling) {
                if (e.Reason == (int)FldEvent.sdFldChoiceListChg) {
                    BuildListItems();
                } else {
                    InitializeCtlv();
                }
            }
        }

        //*********************************************************
        //
        /// <summary>
        /// OnSelectedIndexChanged:     Handle the selected index changed event
        /// </summary>
        /// <param name="e">            Event arguments</param>
        //
        //*********************************************************
        protected override void OnSelectedIndexChanged(System.EventArgs e) {
            if (!m_bFilling) {
                SetFieldValuev();
            }
            base.OnSelectedIndexChanged(e);
        }

        //*********************************************************
        //
        /// <summary>
        /// OnCreateControl:    Handle create control event
        /// </summary>
        //
        //*********************************************************
        protected override void OnCreateControl() {
            RepPP.Application   app;

            app = Application;
            if (app != null) {
                if (DesignMode) {
                    BuildListItems();
                } else {
                    InitializeCtlv();
                    app.ProgInit    += new RepPP.ProgInitEventHandler(this.OnProgInit);
                    app.ProgEnd     += new RepPP.ProgEndEventHandler(this.OnProgEnd);
                    app.FldChange   += new RepPP.FldChangeEventHandler(this.OnFldChange);
                }
            }
            base.OnCreateControl();
        }
		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	} // SCFldListBox
} // namespace
