//*********************************************************
//
// frmMain:     Main form in witch the step forms will be attached
//
// Author:		Maher Najah
//
// (c) 2003 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.IO;
using sdfwwebcsnet;
using System.Web;
using Microsoft.Win32;
using RepPP.VSNetWizard;

namespace RepPP.VSNetWizard.Web
{
	/// <summary>
	/// Main wizard form.
	/// </summary>
	internal class frmMain : frmMainBase {
        private frmStep0                            m_frmStep0;         // step 0 form
        private frmStep1                            m_frmStep1;         // step 1 form
        private frmStep2                            m_frmStep2;         // step 2 form
        private frmStep3                            m_frmStep3;         // step 3 form
        private frmStep4                            m_frmStep4;         // step 4 form
        private frmStep5                            m_frmStep5;         // step 5 form
                   
		private System.ComponentModel.Container     components = null;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public frmMain() {
			InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="vsApp">            Visual studio application</param>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <param name="eLang">            Language</param>
        /// <param name="eMode">            Wizard generation mode</param>
        //
        //******************************************************************
		public frmMain(EnvDTE.DTE       vsApp,
                       string           strPrjName,
                       string           strPrjFullName,
                       ProgramLanguageE eLang,
                       GenerationModeE  eMode) : base (vsApp,
                                                       strPrjName,
                                                       strPrjFullName,
                                                       eLang,
                                                       eMode) {
			InitializeComponent();
            m_frmStep0      = new frmStep0(this);
            m_frmStep1      = new frmStep1(this);
            m_frmStep2      = new frmStep2(this);
            m_frmStep3      = new frmStep3(this);
            m_frmStep4      = new frmStep4(this);
            m_frmStep5      = new frmStep5(this);
            butBack.Enabled = false;
            butNext.Enabled = true;
            m_frmStep0.AttachToMainPanelV();
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) 
        {
            if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
                if (m_frmStep0 != null) {
                    m_frmStep0.Dispose();
                    m_frmStep0 = null;
                }
                if (m_frmStep1 != null) {
                    m_frmStep1.Dispose();
                    m_frmStep1 = null;
                }
                if (m_frmStep2 != null) {
                    m_frmStep2.Dispose();
                    m_frmStep2 = null;
                }
                if (m_frmStep3 != null) {
                    m_frmStep3.Dispose();
                    m_frmStep3 = null;
                }
                if (m_frmStep4 != null) {
                    m_frmStep4.Dispose();
                    m_frmStep4 = null;
                }
                if (m_frmStep5 != null) {
                    m_frmStep5.Dispose();
                    m_frmStep5 = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// HTMLMetaTemplateFile:             HTML meta-template file name
        /// </summary>
        //
        //******************************************************************
        public string HTMLMetaTemplateFile {
            get {
                return(m_frmStep1.HTMLMetaTemplateFile);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenerationMode:     Generation maode(0: group, 1: card, 2: card with selection buffer)
        /// </summary>
        //
        //******************************************************************
        public GenerationModeE GenerationMode {
            get {
                return(m_frmStep3.GenerationMode);
            }
        }

        //*********************************************************     
        //
        /// <summary>
        /// IsHTMLDirEqualsLocalhostb:  Check if the rep++ html dir is equal to localhost
        /// </summary>
        /// <returns>
        /// True:                       Rep++ html dir is equal to localhost
        /// Flase:                      Rep++ html dir isn't equal to localhost
        /// </returns>
        //  
        //*********************************************************     
        private bool IsHTMLDirEqualsLocalhostb() {
            bool        bRetVal = false;
            string      strHtmlDir;
            string      strLocalhostDir = String.Empty;
            RegistryKey key;

            strHtmlDir = m_app.Tool.GetDir(RepPP.DirType.sdDirHTML);
            strHtmlDir = Path.GetDirectoryName(strHtmlDir);
            key        = Registry.LocalMachine.OpenSubKey("Software\\Microsoft\\InetStp");
            strLocalhostDir = key.GetValue("PathWWWRoot", "").ToString();
            key.Close();
            bRetVal = strHtmlDir.ToLower().Equals(strLocalhostDir.ToLower());
            return(bRetVal);
        }

        //*********************************************************     
        //
        /// <summary>
        /// SetProjectNamev:  Set the project name: it should be http://localhost/system/module
        /// </summary>
        //  
        //*********************************************************     
        private void SetProjectNamev() {
            RepPP.Card      crd;
            RepPP.Module    mod;

            crd              = m_app.Cards[m_frmStep3.CardName];
            mod              = crd.Module;
            m_strPrjName     = mod.Name;
            m_strPrjFullName = "http://localhost/" + mod.System + "/" + mod.Name;
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButNextClick:         Handle next button click event
        /// </summary>
        /// <param name="sender">   Sender object   </param>
        /// <param name="e">        Event Arguments </param>
        //  
        //*********************************************************     
        protected override void OnButNextClick(object sender, System.EventArgs e) {
            switch(m_iCurStep) {
            case 0:
                SetStepPanelV(m_frmStep1, true);
                break;
            case 1:
                SetStepPanelV(m_frmStep2, true);
                break;
            case 2:
                if (InitializeRepPP(m_frmStep2.System.ToUpper(),
                                    m_frmStep2.Program.ToUpper(),
                                    m_frmStep2.Connection.ToUpper(),
                                    m_frmStep2.User.ToUpper(),
                                    m_frmStep2.Password,
                                    (RepPP.LanguageCode)m_frmStep2.ConnectionLanguage)) {
                    if (IsHTMLDirEqualsLocalhostb()) {
                        SetStepPanelV(m_frmStep3, true);
                    } else {
                        MessageBox.Show("Rep++ html directory should be equal to http local host directory. Please set\r\n" +
                                        " the rep++ html directory before launching rep++ web application wizard.",
                                        "Rep++ Web Application Wizard",
                                        MessageBoxButtons.OK,
                                        MessageBoxIcon.Error);
                        m_iCurStep--;
                    }
                } else {
                    m_iCurStep--;
                }
                break;
            case 3:
                SetStepPanelV(m_frmStep4, true);
                break;
            case 4:
                SetStepPanelV(m_frmStep5, true);
                butNext.Enabled      = false;
                butCancelFinish.Text = "&Finish";
                if (GenerationMode == GenerationModeE.sdApplication) {
                    SetProjectNamev();
                }
                break;
            }
            m_iCurStep++;
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButBackClick:         Handle back button click event
        /// </summary>
        /// <param name="sender">   Sender object   </param>
        /// <param name="e">        Event Arguments </param>
        //  
        //*********************************************************     
        protected override void OnButBackClick(object sender, System.EventArgs e) {
            switch(m_iCurStep) {
            case 1:
                SetStepPanelV(m_frmStep0, false);
                butBack.Enabled = false;
                break;
            case 2:
                SetStepPanelV(m_frmStep1, false);
                break;
            case 3:
                SetStepPanelV(m_frmStep2, false);
                break;
            case 4:
                SetStepPanelV(m_frmStep3, false);
                break;
            case 5:
                SetStepPanelV(m_frmStep4, false);
                butCancelFinish.Text = "&Cancel";
                break;
            }
            m_iCurStep--;        
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButCancelFinishClick:     Handle cancel/finish button click event
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event Arguments </param>
        //  
        //*********************************************************     
        protected override void OnButCancelFinishClick(object sender, System.EventArgs e) {
            GeneratorBase               generator = null;
            Utility.GenTypedInstParam   genTypedInstParam;

            try {
                if (butCancelFinish.Text == "&Finish") {
                    genTypedInstParam = new Utility.GenTypedInstParam(m_frmStep4.GenTypedInstance,
                                                                      m_frmStep4.Namespace,
                                                                      "",
                                                                      m_frmStep4.CardInstanceBaseClass,
                                                                      m_frmStep4.GroupInstanceBaseClass,
                                                                      m_frmStep4.CardInstanceClassPrefix,
                                                                      m_frmStep4.GroupInstanceClassPrefix,
                                                                      m_frmStep4.GroupInstancePropertyPrefix,
                                                                      m_frmStep4.FieldPropertyPrefix);
                    switch (m_eLang) {
                    case ProgramLanguageE.sdCSharp:
                        generator = new GeneratorForCS(m_vsApp,
                                                       m_app,
                                                       m_frmStep1.HTMLTemplateFolder,
                                                       m_frmStep1.HTMLMetaTemplateFile,
                                                       m_frmStep2.SelectedLanguages,
                                                       genTypedInstParam);
                        break;
                    case ProgramLanguageE.sdVisualBasic:
                        generator = new GeneratorForVB(m_vsApp,
                                                       m_app,
                                                       m_frmStep1.HTMLTemplateFolder,
                                                       m_frmStep1.HTMLMetaTemplateFile,
                                                       m_frmStep2.SelectedLanguages,
                                                       genTypedInstParam);
                        break;
                    default:
                        break;
                    }
                    if (generator != null) {
                        switch (GenerationMode) {
                        case GenerationModeE.sdApplication:
                            generator.GenerateApplicationV(ref m_strPrjName, 
                                                           ref m_strPrjFullName,
                                                           m_frmStep3.CardName,
                                                           m_frmStep3.SBCardName);
                            break;
                        case GenerationModeE.sdItemCard:
                            generator.GenerateCardItemV(m_strPrjName, 
                                                        m_frmStep3.CardName, 
                                                        m_frmStep3.SBCardName);
                            break;
                        case GenerationModeE.sdItemGroup:
                            generator.GenerateGroupItemV(m_strPrjName, m_frmStep3.GroupName);
                            break;
                        default:
                            break;
                        }
                        m_frmStep1.SaveTempFolderV();
                        if (m_frmStep4.GenTypedInstance != Utility.GenTypedInstTypeE.None) {
                            Utility.GenTypedInstanceConfigFile(Utility.GetProject(m_vsApp, m_strPrjName),
                                                               m_frmStep2.System.ToUpper(),
                                                               m_frmStep2.Program.ToUpper(),
                                                               m_frmStep2.Connection.ToUpper(),
                                                               m_frmStep2.User.ToUpper(),
                                                               (RepPP.LanguageCode)m_frmStep2.ConnectionLanguage,
                                                               genTypedInstParam.m_strCardInstClassPrefix,
                                                               genTypedInstParam.m_strGroupInstClassPrefix,
                                                               genTypedInstParam.m_strGroupInstPropPrefix,
                                                               genTypedInstParam.m_strFieldPropPrefix,
                                                               genTypedInstParam.m_strNamespace,
                                                               genTypedInstParam.m_strRootNamespace,
                                                               m_frmStep4.CardInstanceBaseClass,
                                                               m_frmStep4.GroupInstanceBaseClass,
                                                               m_frmStep4.GenTypedInstance == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter);
                            m_frmStep4.SaveTypedInstParam();
                        }
                    }
                }
            } catch (System.Exception ex) {
                MessageBox.Show(ex.ToString());
            } finally {
                Close();
            }
        }
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(frmMain));
            this.SuspendLayout();
            // 
            // pnlMain
            // 
            this.pnlMain.Name = "pnlMain";
            // 
            // butCancelFinish
            // 
            this.butCancelFinish.Name = "butCancelFinish";
            // 
            // butNext
            // 
            this.butNext.Name = "butNext";
            // 
            // butBack
            // 
            this.butBack.Name = "butBack";
            // 
            // pictureBox1
            // 
            this.pictureBox1.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("pictureBox1.BackgroundImage")));
            this.pictureBox1.Name = "pictureBox1";
            // 
            // grpBar
            // 
            this.grpBar.Name = "grpBar";
            // 
            // frmMain
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(554, 326);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "frmMain";
            this.Text = "Rep++ Web Application Wizard";
            this.ResumeLayout(false);

        }
		#endregion
	}
}
